@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2016-2019 Mike Fährmann
+# Copyright 2016-2022 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -9,10 +9,9 @@
 """Extract images from https://www.tumblr.com/"""
 
 from .common import Extractor, Message
-from .. import text, oauth, extractor, exception
+from .. import text, oauth, exception
 from datetime import datetime, timedelta
 import re
-import time
 
 
 def _original_inline_image(url):
@@ -36,15 +35,16 @@ POST_TYPES = frozenset((
 
 BASE_PATTERN = (
     r"(?:tumblr:(?:https?://)?([^/]+)|"
-    r"(?:https?://)?([^.]+\.tumblr\.com))")
+    r"(?:https?://)?([\w-]+\.tumblr\.com))")
 
 
 class TumblrExtractor(Extractor):
     """Base class for tumblr extractors"""
     category = "tumblr"
-    directory_fmt = ("{category}", "{name}")
+    directory_fmt = ("{category}", "{blog_name}")
     filename_fmt = "{category}_{blog_name}_{id}_{num:>02}.{extension}"
     archive_fmt = "{id}_{num}"
+    cookiedomain = None
 
     def __init__(self, match):
         Extractor.__init__(self, match)
@@ -69,7 +69,6 @@ class TumblrExtractor(Extractor):
 
     def items(self):
         blog = None
-        yield Message.Version, 1
 
         for post in self.posts():
             if self.date_min > post["timestamp"]:
@@ -79,10 +78,10 @@ class TumblrExtractor(Extractor):
             if not blog:
                 blog = self.api.info(self.blog)
                 blog["uuid"] = self.blog
-                yield Message.Directory, blog.copy()
 
                 if self.avatar:
                     url = self.api.avatar(self.blog)
+                    yield Message.Directory, {"blog": blog}
                     yield self._prepare_avatar(url, post.copy(), blog)
 
             reblog = "reblogged_from_id" in post
@@ -90,12 +89,12 @@ class TumblrExtractor(Extractor):
                 continue
             post["reblogged"] = reblog
 
-            post["blog"] = blog
-            post["date"] = text.parse_timestamp(post["timestamp"])
-            post["num"] = 0
-
             if "trail" in post:
                 del post["trail"]
+            post["blog"] = blog
+            post["date"] = text.parse_timestamp(post["timestamp"])
+            yield Message.Directory, post
+            post["num"] = 0
 
             if "photos" in post:  # type "photo" or "link"
                 photos = post["photos"]
@@ -108,11 +107,11 @@ class TumblrExtractor(Extractor):
                     del photo["alt_sizes"]
                     yield self._prepare_image(photo["url"], post)
 
-            url = post.get("audio_url")  # type: "audio"
+            url = post.get("audio_url")  # type "audio"
             if url and url.startswith("https://a.tumblr.com/"):
                 yield self._prepare(url, post)
 
-            url = post.get("video_url")  # type: "video"
+            url = post.get("video_url")  # type "video"
             if url:
                 yield self._prepare(_original_video(url), post)
 
@@ -129,12 +128,9 @@ class TumblrExtractor(Extractor):
 
             if self.external:  # external links
                 post["extension"] = None
-                with extractor.blacklist(("tumblr",)):
-                    for key in ("permalink_url", "url"):
-                        url = post.get(key)
-                        if url:
-                            yield Message.Queue, url, post
-                            break
+                url = post.get("permalink_url") or post.get("url")
+                if url:
+                    yield Message.Queue, url, post
 
     def posts(self):
         """Return an iterable containing all relevant posts"""
@@ -156,8 +152,8 @@ class TumblrExtractor(Extractor):
             invalid = types - POST_TYPES
             if invalid:
                 types = types & POST_TYPES
-                self.log.warning('invalid post types: "%s"',
-                                 '", "'.join(sorted(invalid)))
+                self.log.warning("Invalid post types: '%s'",
+                                 "', '".join(sorted(invalid)))
             return types
 
     @staticmethod
@@ -194,7 +190,7 @@ class TumblrExtractor(Extractor):
         return not self.reblogs
 
     def _skip_reblog_same_blog(self, post):
-        return self.blog != post["reblogged_root_uuid"]
+        return self.blog != post.get("reblogged_root_uuid")
 
 
 class TumblrUserExtractor(TumblrExtractor):
@@ -261,9 +257,6 @@ class TumblrPostExtractor(TumblrExtractor):
         ("https://mikf123.tumblr.com/post/167623351559/link-post", {
             "count": 2,
         }),
-        ("https://muyanna.tumblr.com/post/180692431632/answer-post", {
-            "count": 1,
-        }),
         ("https://mikf123.tumblr.com/post/167633596145/video-post", {
             "count": 2,
         }),
@@ -276,9 +269,6 @@ class TumblrPostExtractor(TumblrExtractor):
         ("https://mikf123.tumblr.com/post/181022380064/chat-post", {
             "count": 0,
         }),
-        ("http://pinetre-3.tumblr.com/post/181904381470/via", {
-            "count": 0,  # audio post with "null" as URL (#165)
-        }),
         ("http://ziemniax.tumblr.com/post/109697912859/", {
             "exception": exception.NotFoundError,  # HTML response (#297)
         }),
@@ -302,7 +292,7 @@ class TumblrPostExtractor(TumblrExtractor):
 class TumblrTagExtractor(TumblrExtractor):
     """Extractor for images from a tumblr-user by tag"""
     subcategory = "tag"
-    pattern = BASE_PATTERN + r"/tagged/([^/?&#]+)"
+    pattern = BASE_PATTERN + r"/tagged/([^/?#]+)"
     test = ("http://demo.tumblr.com/tagged/Times%20Square", {
         "pattern": (r"https://\d+\.media\.tumblr\.com/tumblr_[^/_]+_1280.jpg"),
         "count": 1,
@@ -310,7 +300,7 @@ class TumblrTagExtractor(TumblrExtractor):
 
     def __init__(self, match):
         TumblrExtractor.__init__(self, match)
-        self.tag = text.unquote(match.group(3))
+        self.tag = text.unquote(match.group(3).replace("-", " "))
 
     def posts(self):
         return self.api.posts(self.blog, {"tag": self.tag})
@@ -319,7 +309,7 @@ class TumblrTagExtractor(TumblrExtractor):
 class TumblrLikesExtractor(TumblrExtractor):
     """Extractor for images from a tumblr-user's liked posts"""
     subcategory = "likes"
-    directory_fmt = ("{category}", "{name}", "likes")
+    directory_fmt = ("{category}", "{blog_name}", "likes")
     archive_fmt = "f_{blog[name]}_{id}_{num}"
     pattern = BASE_PATTERN + r"/likes"
     test = ("http://mikf123.tumblr.com/likes", {
@@ -407,27 +397,27 @@ class TumblrAPI(oauth.OAuth1API):
             # daily rate limit
             if response.headers.get("x-ratelimit-perday-remaining") == "0":
                 reset = response.headers.get("x-ratelimit-perday-reset")
+                t = (datetime.now() + timedelta(seconds=float(reset))).time()
+
+                self.log.error("Daily API rate limit exceeded")
+
+                api_key = self.api_key or self.session.auth.consumer_key
+                if api_key == self.API_KEY:
+                    self.log.info("Register your own OAuth application and "
+                                  "use its credentials to prevent this error: "
+                                  "https://github.com/mikf/gallery-dl/blob/mas"
+                                  "ter/docs/configuration.rst#extractortumblra"
+                                  "pi-key--api-secret")
+
                 raise exception.StopExtraction(
-                    "Daily API rate limit exceeded: aborting; "
-                    "rate limit will reset at %s", self._to_time(reset),
-                )
+                    "Aborting - Rate limit will reset at %s",
+                    "{:02}:{:02}:{:02}".format(t.hour, t.minute, t.second))
 
             # hourly rate limit
             reset = response.headers.get("x-ratelimit-perhour-reset")
             if reset:
-                self.log.info(
-                    "Hourly API rate limit exceeded; waiting until "
-                    "%s for rate limit reset", self._to_time(reset),
-                )
-                time.sleep(int(reset) + 1)
+                self.log.info("Hourly API rate limit exceeded")
+                self.extractor.wait(seconds=reset)
                 return self._call(blog, endpoint, params)
 
         raise exception.StopExtraction(data)
-
-    @staticmethod
-    def _to_time(reset):
-        try:
-            reset_time = datetime.now() + timedelta(seconds=int(reset))
-        except (ValueError, TypeError):
-            return "?"
-        return reset_time.strftime("%H:%M:%S")
