@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2015-2021 Mike Fährmann
+# Copyright 2015-2022 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -53,9 +53,6 @@ class Job():
                 extr.category = pextr.category
                 extr.subcategory = pextr.subcategory
 
-            # reuse connection adapters
-            extr.session.adapters = pextr.session.adapters
-
         # user-supplied metadata
         kwdict = extr.config("keywords")
         if kwdict:
@@ -501,14 +498,15 @@ class DownloadJob(Job):
         clist = self.extractor.config("whitelist")
         if clist is not None:
             negate = False
+            special = None
         else:
             clist = self.extractor.config("blacklist")
             negate = True
+            special = util.SPECIAL_EXTRACTORS
             if clist is None:
                 clist = (self.extractor.category,)
 
-        return util.build_extractor_filter(
-            clist, negate, util.SPECIAL_EXTRACTORS)
+        return util.build_extractor_filter(clist, negate, special)
 
 
 class SimulationJob(DownloadJob):
